/* Bluefish HTML Editor
 * bluefish.c - main source file
 *
 * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include "config.h"		/* must be before the gdk_imlib !!!!! */

#if HAVE_LIBGDK_IMLIB
#include <gdk_imlib.h>
#endif

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <locale.h>
#include <libintl.h>
#define _(STRING) gettext(STRING)

#include "bluefish.h"
#include "interface.h"
#include "callbacks.h"
#include "commandline.h"
#include "init.h"
#include "debug.h"

mainstruct *main_v;

void modal_fs_ok_clicked (GtkWidget * widget, gpointer window);
void modal_fs_multi_ok_clicked (GtkWidget * widget, gpointer window);
void close_modal_window (GtkWidget * widget, gpointer window);
void file_to_textbox (gchar * filename);


void
cya_later (GtkWidget * widget, gpointer data)
{

  file_close_all (NULL, NULL);
  project_close (NULL, NULL);
  if (g_list_length (main_v->documentlist) > 1)
    {
      if ((g_list_length (main_v->documentlist) == 2)
	  && (main_v->current_document->modified == 0))
	{
	  DEBUG_MSG ("cya_later, doesn't matter, 1 file and not modified\n");
	  gtk_main_quit ();
	}
      else
	{
	  DEBUG_MSG ("cya_later, not all documents are closed\n");
	}
    }
  else
    {
      gtk_main_quit ();
    }
}

int
main (gint argc, gchar * argv[])
{
  parse_commandline(argc, argv);

  /* see if we're running as root, if so, exit with an error unless overriden */
  if (root_override != 1)
    {
      if (geteuid () == 0)
	{
	  g_print (_("Error: cannot run as root (for security reasons)\n"));
	  g_print (_("use -s to override\n"));
	  /* effective uid == 0 and the real uid != 0 means it's suid root */
	  /* give an additional message if this is the case */
	  if (getuid () != 0)
	    {
	      g_print (_("Bluefish is SUID root, this is a security risk!\n"));
	    }
	  exit (EXIT_FAILURE);
	}
    }

  /* for international language support */
  gtk_set_locale ();

  gdk_init (&argc, &argv);

#if HAVE_LIBGDK_IMLIB
  /* Immediately after initialising GDK, Initialise Imlib */
  gdk_imlib_init ();
#endif

  /* create the window widget and start gtk */
  gtk_init (&argc, &argv);

  DEBUG_MSG ("main, 1 g_get_home_dir=%s\n", g_get_home_dir());

  /*initialise the main struct */
  main_v = g_malloc0 (sizeof (mainstruct));

  /* display a splash screen with progress bar
   * also creates the main window, reads the config, 
   * checks directories etc... 
   */
  startup ();
  DEBUG_MSG ("main, startup() finished, going into gtk_main()\n");

  gtk_main (); 

  exit (0);
}

/*
 * Function: change_dir
 * Arguments:
 *      filename - name of file or directory
 * Return value:
 *      FALSE - if error
 *      TRUE - if successful
 * Description:
 *      Will search directory in current path and try change
 *      current path to that directory
 */
gboolean
change_dir (gchar * filename)
{
  gchar *tmpstring;
  gboolean returncode;

  if (!filename)
    return FALSE;
  tmpstring = g_dirname (filename);
  DEBUG_MSG ("change_dir, filename = %s, tmpstring = %s in %p\n",
	     filename, tmpstring, tmpstring);
  returncode = chdir (tmpstring) == 0 ? TRUE : FALSE;
  g_free (tmpstring);
  DEBUG_MSG ("change_dir, after gfree(tmpstring) \n");
  return returncode;
};

/*
 * Function: trunc_on_char
 * Arguments:
 *      string - text string
 *      which_char      - termination character
 * Return value:
 *      pointer same as string
 * Description:
 *      Find which_char and truncate string 'string' on it
 */
gchar *
trunc_on_char (gchar * string, gchar which_char)
{
  gint count;
  for (count = 0; count <= strlen (string); count++)
    {
      if (string[count] == which_char)
	{
	  string[count] = '\0';
	  return string;
	};
    };
  return string;
};

/*
 * Function: strip_filename
 * Arguments:
 *      string - file name with full path
 * Return value:
 *      new allocated string with file name only
 * Description:
 *      select basename
 */
char *
strip_filename (gchar * string)
{
  return g_basename (string);
}

/* this function is useful in changing absolute paths to relative paths */
gchar *
strip_common_path (char *image_fn, char *html_fn)
{
  gchar *tempstr;
  int count, count2, dir_length;

  count = 0;
  tempstr = strrchr (image_fn, '/');
  dir_length = strlen (image_fn) - strlen (tempstr);
  dir_length += 1;
  DEBUG_MSG ("strip_common_path, dir_lenght=%d\n", dir_length);
  while ((strncmp (image_fn, html_fn, count + 1)) == 0)
    {
      count++;
      if (count > dir_length)
	{
	  count = dir_length;
	  break;
	}
    }

  /* contributed by Tony Kemp <tony.kemp@studentmail.newcastle.edu.au> */
  while (image_fn[count - 1] != '/')
    count--;


  DEBUG_MSG ("strip_common_path, equal count = %d\n", count);
  count2 = strlen (image_fn);
  tempstr = g_malloc (count2 - count + 2);
  memcpy (tempstr, &image_fn[count], count2 - count + 2);
  DEBUG_MSG ("strip_common_path, tempstr= %s, should be %d long\n",
	     tempstr, count2 - count);
  return tempstr;
  /* the function that calls this one must use gfree() somewhere!! */
}



void
window_close (GtkWidget * button, gpointer windowname)
{
  gtk_signal_handlers_destroy (GTK_OBJECT (windowname));
  gtk_grab_remove (windowname);
  gtk_widget_destroy (windowname);
}

gint
file_exists_and_readable (gchar * filename)
{
  struct stat naamstat;

  if (filename == NULL)
    {
      DEBUG_MSG ("file_exists_and_readable, filename == NULL!!!!\n");
      return 0;
    }

  if (strlen (filename) < 2)
    {
      DEBUG_MSG ("file_exists_and_readable, strlen(filename) < 2!!!!\n");
      return 0;
    }
  DEBUG_MSG
    ("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n",
     filename, filename, strlen (filename));
  if ((stat (filename, &naamstat) == -1) && (errno == ENOENT))
    {
/*              if(naamstat.st_mode & S_IREAD) { */
      return (0);
/*              }
   return(0); */
    }
  else
    {
      return (1);
    }
}


/* forces everything to be flushed */
void
flush_queue (void)
{
  while (gtk_events_pending ())
    {
      gtk_main_iteration ();
    }
  while (gdk_events_pending ())
    {
      gdk_flush ();
    }
}

/*
 * These are the functions to return a file using modal windows,
 * just give a call to return_file() and you get a filename, or
 * you get NULL if non selected
 */

/* just for the modal file selection, a glocal gchar pointer */
gchar *filename_to_return;

gchar *
return_file (gchar * set_file)
{

  GtkWidget *fs;
  gchar *tmp;
  DEBUG_MSG ("return file, started\n");
  fs = gtk_file_selection_new (_("Select file"));

  gtk_signal_connect (GTK_OBJECT (fs), "destroy", close_modal_window, fs);
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) modal_fs_ok_clicked, fs);
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
		      "clicked", (GtkSignalFunc) close_modal_window,
		      GTK_OBJECT (fs));
  if (set_file != NULL)
    {
      gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), set_file);
    }
  gtk_widget_show (fs);
  gtk_grab_add (GTK_WIDGET (fs));
  gtk_widget_realize (GTK_WIDGET (fs));
  gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (main_v->main_window));
  gtk_main ();
  tmp = filename_to_return;
  filename_to_return = NULL;
  return tmp;
}

/*
 * These are the functions to return multiple file using modal windows,
 * just give a call to return_file() and you get the filenames, or
 * you get NULL if non selected
 */

void
return_files (void)
{

  GtkWidget *fs;
  DEBUG_MSG ("return file, started\n");
  fs = gtk_file_selection_new (_("Select file"));

  gtk_signal_connect (GTK_OBJECT (fs), "destroy", close_modal_window, fs);
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
		      "clicked", (GtkSignalFunc) modal_fs_multi_ok_clicked, fs);
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button),
		      "clicked", (GtkSignalFunc) close_modal_window,
		      GTK_OBJECT (fs));
  gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION 
              (fs)->file_list), GTK_SELECTION_EXTENDED);
  gtk_widget_show (fs);
  gtk_grab_add (GTK_WIDGET (fs));
  gtk_widget_realize (GTK_WIDGET (fs));
  gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (main_v->main_window));
  gtk_main ();
  return;
}


/* The following is the multi-file loader and displayer. heavily "inspired"
   by the GIMP Loader */

void
modal_fs_multi_ok_clicked (GtkWidget * widget, gpointer data)
{
  GtkFileSelection *fs;
  gchar *filename;
  gchar *totalfilename=NULL;
  gchar *raw_filename;
  gchar *mfilename=NULL;
  gchar *filedirname;
  struct stat buf;
  int err;

  fs = GTK_FILE_SELECTION (data);
  filename = gtk_file_selection_get_filename (fs);
  raw_filename = gtk_entry_get_text (GTK_ENTRY(fs->selection_entry));

  g_assert (filename && raw_filename);

  if (strlen (raw_filename) == 0)
    return;

  err = stat (filename, &buf);

  if (err == 0 && (buf.st_mode & S_IFDIR))
    {
      GString *s = g_string_new (filename);
      if (s->str[s->len - 1] != G_DIR_SEPARATOR)
        {
          g_string_append_c (s, G_DIR_SEPARATOR);
        }
      gtk_file_selection_set_filename (fs, s->str);
      g_string_free (s, TRUE);
      return;
    }

  gtk_widget_set_sensitive (GTK_WIDGET (fs), FALSE);

  if (err)
    filename = raw_filename;

    DEBUG_MSG ("file_open, returned file %s\n", filename);
    main_v->current_document = new_document ();
    main_v->current_document->filename= g_strdup(filename);
    file_to_textbox (main_v->current_document->filename);
    gtk_label_set (GTK_LABEL (main_v->current_document->tab_label), strip_filename (main_v->current_document->filename));


  /*
   * Now deal with multiple selections from the filesel clist
   */


  /*  filedirname = fs->history_list->data;*/
  
  filedirname = g_dirname (filename);

  {
    GList *row = GTK_CLIST(fs->file_list)->row_list;
    gint rownum = 0;
    gchar* temp;

    while (row)
      {
	if (GTK_CLIST_ROW(row)->state == GTK_STATE_SELECTED)
	  {
	    if (gtk_clist_get_cell_type(GTK_CLIST(fs->file_list),
					rownum, 0) == GTK_CELL_TEXT)
	      {
		gtk_clist_get_text (GTK_CLIST(fs->file_list),
				    rownum, 0, &temp);
		
		/* When doing multiple selections, the name
		 * of the first item touched with the cursor will
		 * become the text-field default - and we don't
		 * want to load that twice.
		 */
		if (strcmp(temp, raw_filename)==0)
		  {
		    goto next_iter;
		  }

		mfilename = g_strdup (temp);
		
		err = stat (mfilename, &buf);
		
		totalfilename = g_strconcat(filedirname,"/",NULL);
		totalfilename = g_strconcat(totalfilename,mfilename,NULL);
		
		if (! (err == 0 && (buf.st_mode & S_IFDIR)))
		  { /* Is not directory. */
		    
           DEBUG_MSG ("file_open, returned file %s\n", mfilename);
           main_v->current_document = new_document ();
           main_v->current_document->filename=g_strdup(totalfilename);
           file_to_textbox (main_v->current_document->filename);
           gtk_label_set (GTK_LABEL (main_v->current_document->tab_label), strip_filename (main_v->current_document->filename));
		  }
	      }
	  }

      next_iter:
	if (mfilename)
	  {
	    g_free (mfilename);
	    mfilename = NULL;
	  }
	rownum++;
	row = g_list_next (row);
      }
  }

  if (filedirname)
    g_free (filedirname);
    gtk_main_quit ();
  gtk_grab_remove (GTK_WIDGET (data));
  gtk_signal_handlers_destroy (data);
  gtk_widget_destroy (GTK_WIDGET (data));

}

void
modal_fs_ok_clicked (GtkWidget * widget, gpointer window)
{
  filename_to_return = g_strdup(gtk_file_selection_get_filename
					(GTK_FILE_SELECTION (window)));
  gtk_main_quit ();
  gtk_grab_remove (GTK_WIDGET (window));
  gtk_signal_handlers_destroy (window);
  gtk_widget_destroy (GTK_WIDGET (window));
}

void
close_modal_window (GtkWidget * widget, gpointer window)
{
  if (filename_to_return != NULL)
    {
      g_free (filename_to_return);
      filename_to_return = NULL;
    }
  gtk_main_quit ();
  gtk_grab_remove (GTK_WIDGET (window));
  gtk_signal_handlers_destroy (window);
  gtk_widget_destroy (GTK_WIDGET (window));
}
